'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

const chokidar = require('chokidar');
const colorette = require('colorette');
const path = require('path');
const matter = require('gray-matter');
const fsExtra = require('fs-extra');
const fg = require('fast-glob');
const minimist = require('minimist');
const utils = require('@ruabick/utils');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e["default"] : e; }

const chokidar__default = /*#__PURE__*/_interopDefaultLegacy(chokidar);
const matter__default = /*#__PURE__*/_interopDefaultLegacy(matter);
const fsExtra__default = /*#__PURE__*/_interopDefaultLegacy(fsExtra);
const fg__default = /*#__PURE__*/_interopDefaultLegacy(fg);
const minimist__default = /*#__PURE__*/_interopDefaultLegacy(minimist);

const LOG_PREFIX = "[vue-hooks-plus:gen-temp]";
const removeFile = (file) => {
  fsExtra__default.remove(file);
  console.log(`${LOG_PREFIX} ${colorette.green("remove")}  ${file}`);
};

async function handleCopy(dir, path$1, {
  options,
  localeConfigs
}) {
  const { tempDir } = options;
  let destPath;
  if (!path$1.endsWith(".md")) {
    destPath = path.join(tempDir, path$1.replace(new RegExp(`^${dir}`), ""));
    await fsExtra__default.copy(path$1, destPath);
  } else {
    const { finnalPath, finnalContent } = await resolveFrontmatter(
      path$1,
      tempDir,
      dir
    );
    const fileInLangDir = handleLangSuffix(finnalPath, localeConfigs);
    destPath = path.join(tempDir, fileInLangDir);
    await fsExtra__default.ensureFile(destPath);
    await fsExtra__default.writeFile(destPath, finnalContent);
  }
  console.log(`${LOG_PREFIX} ${colorette.green("copy")} ${path$1} \u2192 ${destPath}`);
}
async function resolveFrontmatter(path$1, tempDir, dir) {
  const originalContent = await fsExtra__default.readFile(path$1, "utf-8");
  const { content, data: frontmatter } = matter__default(originalContent);
  const realPath = path$1;
  let finnalPath;
  let mappingPath = frontmatter.mapping?.path ?? frontmatter.map?.path;
  if (mappingPath) {
    if (!mappingPath.endsWith(".md")) {
      mappingPath = path.join(mappingPath, path.basename(path$1));
    }
    finnalPath = mappingPath;
  } else {
    finnalPath = path$1.replace(new RegExp(`^${dir}`), "");
  }
  const str = path$1;
  const isCN = str.includes("zh-CN");
  const base = "https://github.com/InhiblabCore/vue-hooks-plus/tree/master/packages/hooks/";
  const sourceShow = frontmatter?.source?.show ?? true;
  const sourcePath = frontmatter?.source?.path ?? `${base}${str.slice(0, str.lastIndexOf("/"))}/index.ts`;
  const sourceDocPath = frontmatter?.source?.docPath ?? `${base}${path$1}`;
  const sourceDemoPath = frontmatter?.source?.demoPath ?? `${base}${str.slice(0, str.lastIndexOf("/"))}/demo`;
  const showSource = frontmatter?.source?.showSource ?? true;
  const showDocs = frontmatter?.source?.showDocs ?? true;
  const showDemo = frontmatter?.source?.showDemo ?? true;
  const links = [
    showSource ? [isCN ? "\u6E90\u7801" : "Source", sourcePath] : void 0,
    showDocs ? [isCN ? "\u6587\u6863" : "Docs", sourceDocPath] : void 0,
    showDemo ? [isCN ? "\u793A\u4F8B" : "Demo", sourceDemoPath] : void 0
  ].filter((i) => i).map((i) => `[${i[0]}](${i[1]})`).join(" \u2022 ");
  const sourceSection = `## Source

${links}
`;
  const processContent = mappingPath && sourceShow ? `${content}

${sourceSection}` : content;
  const finnalContent = matter__default.stringify(processContent, {
    ...frontmatter ?? {},
    realPath
  });
  return {
    finnalPath,
    finnalContent
  };
}
function handleLangSuffix(path$1, localeConfigs) {
  const { defaultLang, langToPathMap } = localeConfigs;
  if (!Object.keys(langToPathMap).length) {
    return path$1;
  }
  const fileName = path.basename(path$1);
  const dir = path.dirname(path$1);
  const fileNameWithoutMd = fileName.replace(/\.md$/, "");
  const fileExtname = path.extname(fileNameWithoutMd);
  const langSuffix = fileExtname.slice(1) || defaultLang;
  const langPath = langToPathMap[langSuffix];
  if (!langPath) {
    console.log(
      colorette.yellow(
        `${fileName} has a ${fileExtname} suffix. But ${langSuffix} not defined in locales`
      ) + colorette.dim(` .vitepress.config.js`)
    );
    return path$1;
  }
  const fileNameWithoutLangSuffix = fileNameWithoutMd.slice(0, -fileExtname.length || void 0) + ".md";
  return path.join(langPath, dir, fileNameWithoutLangSuffix);
}

async function copyDocs(options, localeConfigs) {
  const { docsDir, initial } = options;
  const globSource = `${docsDir}/**`;
  const copyFile = (file) => handleCopy(docsDir, file, { options, localeConfigs });
  if (initial) {
    const files = await fg__default(globSource, { dot: true, cwd: process.cwd() });
    await Promise.all(files.map(copyFile));
  } else {
    chokidar__default.watch(globSource, {
      cwd: process.cwd(),
      ignored: [],
      ignoreInitial: !initial,
      persistent: !initial
    }).on("change", copyFile).on("add", copyFile).on("unlink", (file) => removeFile(file));
  }
}

async function copySrcMd(options, localeConfigs) {
  const { srcDir, initial } = options;
  const copyFile = (file) => handleCopy(srcDir, file, { options, localeConfigs });
  const globSource = `${srcDir}/**/*.md`;
  if (initial) {
    const mdFiles = await fg__default(globSource, {
      dot: true,
      cwd: process.cwd(),
      ignore: ["**/node_modules/**"]
    });
    await Promise.all(mdFiles.map(copyFile));
  } else {
    chokidar__default.watch(globSource, {
      cwd: process.cwd(),
      ignored: [/node_modules/],
      ignoreInitial: true
    }).on("change", copyFile).on("add", copyFile).on("unlink", (file) => removeFile(file));
  }
}

const TempDir = ".docs";
const DocsDir = "docs";
const SrcDir = "src";
const DefaultOptions = {
  srcDir: SrcDir,
  tempDir: TempDir,
  docsDir: DocsDir,
  initial: false,
  lang: "zh-CN"
};

async function genTempDocs(root, inputOptions) {
  const options = {
    ...DefaultOptions,
    ...inputOptions
  };
  const localeConfigs = await utils.resolveLocaleConfigs(root);
  if (options.initial) {
    await fsExtra__default.remove(options.tempDir);
    console.log(`${LOG_PREFIX} remove ${options.tempDir}.`);
  }
  console.log(`${LOG_PREFIX} ${colorette.cyan(`Start generate ${options.tempDir}...`)}`);
  await Promise.all([
    copyDocs(options, localeConfigs),
    copySrcMd(options, localeConfigs)
  ]);
  if (options.initial) {
    console.log(`${LOG_PREFIX} ${colorette.cyan("Initial scan complete.")}`);
    process.exit(0);
  } else {
    console.warn(
      `${colorette.cyan(
        `${LOG_PREFIX} watching ${options.docsDir} and ${options.srcDir}/**/*.md files changes...`
      )}`
    );
  }
}
async function initialScan() {
  const argv = minimist__default(process.argv.slice(2));
  const root = argv._[0];
  genTempDocs(root, {
    initial: true,
    srcDir: argv.srcDir ?? DefaultOptions.srcDir
  });
}
function genTemp() {
  return {
    name: "vite-plugin-gen-temp",
    config: async (config) => {
      await genTempDocs(config.root, { initial: false });
    }
  };
}

exports.genTemp = genTemp;
exports.initialScan = initialScan;
