import chokidar from 'chokidar';
import { green, yellow, dim, cyan } from 'colorette';
import { join, basename, dirname, extname } from 'path';
import matter from 'gray-matter';
import fsExtra from 'fs-extra';
import fg from 'fast-glob';
import minimist from 'minimist';
import { resolveLocaleConfigs } from '@ruabick/utils';

const LOG_PREFIX = "[vue-hooks-plus:gen-temp]";
const removeFile = (file) => {
  fsExtra.remove(file);
  console.log(`${LOG_PREFIX} ${green("remove")}  ${file}`);
};

async function handleCopy(dir, path, {
  options,
  localeConfigs
}) {
  const { tempDir } = options;
  let destPath;
  if (!path.endsWith(".md")) {
    destPath = join(tempDir, path.replace(new RegExp(`^${dir}`), ""));
    await fsExtra.copy(path, destPath);
  } else {
    const { finnalPath, finnalContent } = await resolveFrontmatter(
      path,
      tempDir,
      dir
    );
    const fileInLangDir = handleLangSuffix(finnalPath, localeConfigs);
    destPath = join(tempDir, fileInLangDir);
    await fsExtra.ensureFile(destPath);
    await fsExtra.writeFile(destPath, finnalContent);
  }
  console.log(`${LOG_PREFIX} ${green("copy")} ${path} \u2192 ${destPath}`);
}
async function resolveFrontmatter(path, tempDir, dir) {
  const originalContent = await fsExtra.readFile(path, "utf-8");
  const { content, data: frontmatter } = matter(originalContent);
  const realPath = path;
  let finnalPath;
  let mappingPath = frontmatter.mapping?.path ?? frontmatter.map?.path;
  if (mappingPath) {
    if (!mappingPath.endsWith(".md")) {
      mappingPath = join(mappingPath, basename(path));
    }
    finnalPath = mappingPath;
  } else {
    finnalPath = path.replace(new RegExp(`^${dir}`), "");
  }
  const str = path;
  const isCN = str.includes("zh-CN");
  const base = "https://github.com/InhiblabCore/vue-hooks-plus/tree/master/packages/hooks/";
  const sourceShow = frontmatter?.source?.show ?? true;
  const sourcePath = frontmatter?.source?.path ?? `${base}${str.slice(0, str.lastIndexOf("/"))}/index.ts`;
  const sourceDocPath = frontmatter?.source?.docPath ?? `${base}${path}`;
  const sourceDemoPath = frontmatter?.source?.demoPath ?? `${base}${str.slice(0, str.lastIndexOf("/"))}/demo`;
  const showSource = frontmatter?.source?.showSource ?? true;
  const showDocs = frontmatter?.source?.showDocs ?? true;
  const showDemo = frontmatter?.source?.showDemo ?? true;
  const links = [
    showSource ? [isCN ? "\u6E90\u7801" : "Source", sourcePath] : void 0,
    showDocs ? [isCN ? "\u6587\u6863" : "Docs", sourceDocPath] : void 0,
    showDemo ? [isCN ? "\u793A\u4F8B" : "Demo", sourceDemoPath] : void 0
  ].filter((i) => i).map((i) => `[${i[0]}](${i[1]})`).join(" \u2022 ");
  const sourceSection = `## Source

${links}
`;
  const processContent = mappingPath && sourceShow ? `${content}

${sourceSection}` : content;
  const finnalContent = matter.stringify(processContent, {
    ...frontmatter ?? {},
    realPath
  });
  return {
    finnalPath,
    finnalContent
  };
}
function handleLangSuffix(path, localeConfigs) {
  const { defaultLang, langToPathMap } = localeConfigs;
  if (!Object.keys(langToPathMap).length) {
    return path;
  }
  const fileName = basename(path);
  const dir = dirname(path);
  const fileNameWithoutMd = fileName.replace(/\.md$/, "");
  const fileExtname = extname(fileNameWithoutMd);
  const langSuffix = fileExtname.slice(1) || defaultLang;
  const langPath = langToPathMap[langSuffix];
  if (!langPath) {
    console.log(
      yellow(
        `${fileName} has a ${fileExtname} suffix. But ${langSuffix} not defined in locales`
      ) + dim(` .vitepress.config.js`)
    );
    return path;
  }
  const fileNameWithoutLangSuffix = fileNameWithoutMd.slice(0, -fileExtname.length || void 0) + ".md";
  return join(langPath, dir, fileNameWithoutLangSuffix);
}

async function copyDocs(options, localeConfigs) {
  const { docsDir, initial } = options;
  const globSource = `${docsDir}/**`;
  const copyFile = (file) => handleCopy(docsDir, file, { options, localeConfigs });
  if (initial) {
    const files = await fg(globSource, { dot: true, cwd: process.cwd() });
    await Promise.all(files.map(copyFile));
  } else {
    chokidar.watch(globSource, {
      cwd: process.cwd(),
      ignored: [],
      ignoreInitial: !initial,
      persistent: !initial
    }).on("change", copyFile).on("add", copyFile).on("unlink", (file) => removeFile(file));
  }
}

async function copySrcMd(options, localeConfigs) {
  const { srcDir, initial } = options;
  const copyFile = (file) => handleCopy(srcDir, file, { options, localeConfigs });
  const globSource = `${srcDir}/**/*.md`;
  if (initial) {
    const mdFiles = await fg(globSource, {
      dot: true,
      cwd: process.cwd(),
      ignore: ["**/node_modules/**"]
    });
    await Promise.all(mdFiles.map(copyFile));
  } else {
    chokidar.watch(globSource, {
      cwd: process.cwd(),
      ignored: [/node_modules/],
      ignoreInitial: true
    }).on("change", copyFile).on("add", copyFile).on("unlink", (file) => removeFile(file));
  }
}

const TempDir = ".docs";
const DocsDir = "docs";
const SrcDir = "src";
const DefaultOptions = {
  srcDir: SrcDir,
  tempDir: TempDir,
  docsDir: DocsDir,
  initial: false,
  lang: "zh-CN"
};

async function genTempDocs(root, inputOptions) {
  const options = {
    ...DefaultOptions,
    ...inputOptions
  };
  const localeConfigs = await resolveLocaleConfigs(root);
  if (options.initial) {
    await fsExtra.remove(options.tempDir);
    console.log(`${LOG_PREFIX} remove ${options.tempDir}.`);
  }
  console.log(`${LOG_PREFIX} ${cyan(`Start generate ${options.tempDir}...`)}`);
  await Promise.all([
    copyDocs(options, localeConfigs),
    copySrcMd(options, localeConfigs)
  ]);
  if (options.initial) {
    console.log(`${LOG_PREFIX} ${cyan("Initial scan complete.")}`);
    process.exit(0);
  } else {
    console.warn(
      `${cyan(
        `${LOG_PREFIX} watching ${options.docsDir} and ${options.srcDir}/**/*.md files changes...`
      )}`
    );
  }
}
async function initialScan() {
  const argv = minimist(process.argv.slice(2));
  const root = argv._[0];
  genTempDocs(root, {
    initial: true,
    srcDir: argv.srcDir ?? DefaultOptions.srcDir
  });
}
function genTemp() {
  return {
    name: "vite-plugin-gen-temp",
    config: async (config) => {
      await genTempDocs(config.root, { initial: false });
    }
  };
}

export { genTemp, initialScan };
